﻿using System.Net;

namespace Netina.Domain.Entities.Products;

public partial class Product
{
    public static Product Create(string persianName,
        string englishName,
        string summery,
        string expertCheck,
        string tags,
        string warranty,
        bool beDisplayed,
        double cost, 
        double packingCost,
        bool hasExpressDelivery,
        int stock,
        int maxOrderCount,
        Guid brandId,
        Guid categoryId, Guid authorId)
    {
        var slug = WebUtility.UrlEncode(persianName.Replace(' ', '-'));
        return new Product(
            persianName, 
            englishName,
            slug,
            summery, 
            expertCheck, 
            tags, 
            warranty, 
            beDisplayed,
            cost, 
            packingCost,
            stock,
            hasExpressDelivery,
            maxOrderCount,
            stock > 0,
            false,
            brandId,
            categoryId,authorId);
    }

    public void AddRate(float rate)
    {
        Rate = ((Rate * ReviewCount) + rate) / (ReviewCount + 1);
        ReviewCount++;
    }

    public void RemoveRate(float rate)
    {

        Rate = ((Rate * ReviewCount) - rate) / (ReviewCount - 1);
        ReviewCount--;
    }

    public ProductStorageFile AddFile(string name, string fileLocation, string fileName, bool isHeader, bool isPrimary, StorageFileType fileType)
    {
        var ent = ProductStorageFile.Create(name, fileLocation, fileName, isHeader, isPrimary, fileType, Id);
        Files.Add(ent);
        return ent;
    }

    public Specification AddSpecification(string title, string detail,string value, bool isFeature, Guid parentId)
    {
        var ent = Specification.Create(title, value, detail, isFeature, Id, parentId);
        Specifications.Add(ent);
        return ent;
    }

    public void AddMetaTag(string key, string value)
        => MetaTags.Add(ProductMetaTag.Create(key, value, Id));
}

public partial class ProductComment
{
    public static ProductComment Create(string title, string content, float rate, bool isBuyer, bool isAdmin, Guid userId, Guid productId)
        => new(title, content, rate, isBuyer, isAdmin, userId, productId);
}
public partial class ProductMetaTag
{
    public static ProductMetaTag Create(string key, string value, Guid productId)
        => new ProductMetaTag(key, value, productId);
}

public partial class ProductStorageFile
{
    public static ProductStorageFile Create(string name, string fileLocation, string fileName, bool isHeader,
        bool isPrimary, StorageFileType fileType, Guid productId)
    {
        return new ProductStorageFile(name, fileLocation, fileName, isHeader, isPrimary, fileType, productId);
    }
}



public partial class Specification
{
    public static Specification Create(string title,string value, string detail, bool isFeature, Guid productId, Guid parentId)
    {
        return new Specification(title, detail, value, isFeature, productId, parentId);
    }
}

public partial class SubProduct
{
    public static SubProduct Create(
        Guid parentId,
        ProductDiversity diversity,
        string diversityValue,
        string diversityDescription,
        string persianName,
        string englishName,
        string summery,
        string slug,
        string expertCheck,
        string tags,
        string warranty,
        bool beDisplayed,
        double cost,
        double packingCost,
        bool hasExpressDelivery,
        int stock,
        int maxOrderCount,
        Guid brandId,
        Guid categoryId,
        Guid authorId)
    {
        return new SubProduct(
            parentId,
            diversity,
            diversityValue,
            diversityDescription,
            persianName,
            englishName,
            slug,
            summery,
            expertCheck,
            tags,
            warranty,
            beDisplayed,
            cost,
            packingCost,
            stock,
            hasExpressDelivery,
            maxOrderCount,
            stock > 0,
            brandId,
            categoryId,
            authorId);
    }
}