using System;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Expressions;
using Mapster.Models;
using Netina.Domain.Dtos.LargDtos;
using Netina.Domain.Dtos.SmallDtos;
using Netina.Domain.Entities.ProductCategories;

namespace Netina.Domain.Mappers
{
    public static partial class ProductCategoryMapper
    {
        public static ProductCategory AdaptToProductCategory(this ProductCategoryLDto p1)
        {
            return p1 == null ? null : new ProductCategory()
            {
                Name = p1.Name,
                Slug = p1.Slug,
                Description = p1.Description,
                IsMain = p1.IsMain,
                ParentId = (Guid?)p1.ParentId,
                Parent = new ProductCategory()
                {
                    Name = p1.ParentName,
                    Id = p1.ParentId
                },
                MetaTags = funcMain1(p1.MetaTags),
                Files = funcMain2(p1.Files),
                Id = p1.Id,
                CreatedAt = p1.CreatedAt
            };
        }
        public static ProductCategory AdaptTo(this ProductCategoryLDto p4, ProductCategory p5)
        {
            if (p4 == null)
            {
                return null;
            }
            ProductCategory result = p5 ?? new ProductCategory();
            
            result.Name = p4.Name;
            result.Slug = p4.Slug;
            result.Description = p4.Description;
            result.IsMain = p4.IsMain;
            result.ParentId = (Guid?)p4.ParentId;
            result.Parent = funcMain3(new Never(), result.Parent, p4);
            result.MetaTags = funcMain4(p4.MetaTags, result.MetaTags);
            result.Files = funcMain5(p4.Files, result.Files);
            result.Id = p4.Id;
            result.CreatedAt = p4.CreatedAt;
            return result;
            
        }
        public static Expression<Func<ProductCategoryLDto, ProductCategory>> ProjectToProductCategory => p12 => new ProductCategory()
        {
            Name = p12.Name,
            Slug = p12.Slug,
            Description = p12.Description,
            IsMain = p12.IsMain,
            ParentId = (Guid?)p12.ParentId,
            Parent = new ProductCategory()
            {
                Name = p12.ParentName,
                Id = p12.ParentId
            },
            MetaTags = p12.MetaTags.Select<MetaTagSDto, ProductCategoryMetaTag>(p13 => new ProductCategoryMetaTag()
            {
                Type = p13.Type,
                Value = p13.Value,
                Id = p13.Id,
                CreatedAt = p13.CreatedAt
            }).ToList<ProductCategoryMetaTag>(),
            Files = p12.Files.Select<StorageFileSDto, ProductCategoryStorageFile>(p14 => new ProductCategoryStorageFile()
            {
                Name = p14.Name,
                FileLocation = p14.FileLocation,
                FileName = p14.FileName,
                IsHeader = p14.IsHeader,
                IsPrimary = p14.IsPrimary,
                FileType = p14.FileType,
                Id = p14.Id,
                CreatedAt = p14.CreatedAt
            }).ToList<ProductCategoryStorageFile>(),
            Id = p12.Id,
            CreatedAt = p12.CreatedAt
        };
        public static ProductCategoryLDto AdaptToLDto(this ProductCategory p15)
        {
            return p15 == null ? null : new ProductCategoryLDto()
            {
                Name = p15.Name,
                Description = p15.Description,
                ParentId = p15.ParentId == null ? default(Guid) : (Guid)p15.ParentId,
                ParentName = p15.Parent != null ? p15.Parent.Name : string.Empty,
                Slug = p15.Slug,
                IsMain = p15.IsMain,
                Files = funcMain6(p15.Files),
                MetaTags = funcMain7(p15.MetaTags),
                Id = p15.Id,
                CreatedAt = p15.CreatedAt
            };
        }
        public static ProductCategoryLDto AdaptTo(this ProductCategory p18, ProductCategoryLDto p19)
        {
            if (p18 == null)
            {
                return null;
            }
            ProductCategoryLDto result = p19 ?? new ProductCategoryLDto();
            
            result.Name = p18.Name;
            result.Description = p18.Description;
            result.ParentId = p18.ParentId == null ? default(Guid) : (Guid)p18.ParentId;
            result.ParentName = p18.Parent != null ? p18.Parent.Name : string.Empty;
            result.Slug = p18.Slug;
            result.IsMain = p18.IsMain;
            result.Files = funcMain8(p18.Files, result.Files);
            result.MetaTags = funcMain9(p18.MetaTags, result.MetaTags);
            result.Id = p18.Id;
            result.CreatedAt = p18.CreatedAt;
            return result;
            
        }
        public static Expression<Func<ProductCategory, ProductCategoryLDto>> ProjectToLDto => p24 => new ProductCategoryLDto()
        {
            Name = p24.Name,
            Description = p24.Description,
            ParentId = p24.ParentId == null ? default(Guid) : (Guid)p24.ParentId,
            ParentName = p24.Parent != null ? p24.Parent.Name : string.Empty,
            Slug = p24.Slug,
            IsMain = p24.IsMain,
            Files = p24.Files.Select<ProductCategoryStorageFile, StorageFileSDto>(p25 => new StorageFileSDto()
            {
                Name = p25.Name,
                FileLocation = p25.FileLocation,
                FileName = p25.FileName,
                IsHeader = p25.IsHeader,
                IsPrimary = p25.IsPrimary,
                FileType = p25.FileType,
                Id = p25.Id
            }).ToList<StorageFileSDto>(),
            MetaTags = p24.MetaTags.Select<ProductCategoryMetaTag, MetaTagSDto>(p26 => new MetaTagSDto()
            {
                Type = p26.Type,
                Value = p26.Value,
                Id = p26.Id,
                CreatedAt = p26.CreatedAt
            }).ToList<MetaTagSDto>(),
            Id = p24.Id,
            CreatedAt = p24.CreatedAt
        };
        public static ProductCategory AdaptToProductCategory(this ProductCategorySDto p27)
        {
            return p27 == null ? null : new ProductCategory()
            {
                Name = p27.Name,
                Slug = p27.Slug,
                Description = p27.Description,
                IsMain = p27.IsMain,
                ParentId = (Guid?)p27.ParentId,
                Parent = new ProductCategory()
                {
                    Name = p27.ParentName,
                    Id = p27.ParentId
                },
                Id = p27.Id,
                CreatedAt = p27.CreatedAt
            };
        }
        public static ProductCategory AdaptTo(this ProductCategorySDto p28, ProductCategory p29)
        {
            if (p28 == null)
            {
                return null;
            }
            ProductCategory result = p29 ?? new ProductCategory();
            
            result.Name = p28.Name;
            result.Slug = p28.Slug;
            result.Description = p28.Description;
            result.IsMain = p28.IsMain;
            result.ParentId = (Guid?)p28.ParentId;
            result.Parent = funcMain10(new Never(), result.Parent, p28);
            result.Id = p28.Id;
            result.CreatedAt = p28.CreatedAt;
            return result;
            
        }
        public static ProductCategorySDto AdaptToSDto(this ProductCategory p32)
        {
            return p32 == null ? null : new ProductCategorySDto()
            {
                Name = p32.Name,
                Description = p32.Description,
                Slug = p32.Slug,
                IsMain = p32.IsMain,
                ParentId = p32.ParentId == null ? default(Guid) : (Guid)p32.ParentId,
                ParentName = p32.Parent != null ? p32.Parent.Name : string.Empty,
                MainImage = p32.Files.FirstOrDefault<ProductCategoryStorageFile>(funcMain11) != null ? p32.Files.FirstOrDefault<ProductCategoryStorageFile>(funcMain12).FileLocation : (p32.Files.Count > 0 ? p32.Files.FirstOrDefault<ProductCategoryStorageFile>().FileLocation : string.Empty),
                Id = p32.Id,
                CreatedAt = p32.CreatedAt
            };
        }
        public static ProductCategorySDto AdaptTo(this ProductCategory p33, ProductCategorySDto p34)
        {
            if (p33 == null)
            {
                return null;
            }
            ProductCategorySDto result = p34 ?? new ProductCategorySDto();
            
            result.Name = p33.Name;
            result.Description = p33.Description;
            result.Slug = p33.Slug;
            result.IsMain = p33.IsMain;
            result.ParentId = p33.ParentId == null ? default(Guid) : (Guid)p33.ParentId;
            result.ParentName = p33.Parent != null ? p33.Parent.Name : string.Empty;
            result.MainImage = p33.Files.FirstOrDefault<ProductCategoryStorageFile>(funcMain11) != null ? p33.Files.FirstOrDefault<ProductCategoryStorageFile>(funcMain12).FileLocation : (p33.Files.Count > 0 ? p33.Files.FirstOrDefault<ProductCategoryStorageFile>().FileLocation : string.Empty);
            result.Id = p33.Id;
            result.CreatedAt = p33.CreatedAt;
            return result;
            
        }
        public static Expression<Func<ProductCategory, ProductCategorySDto>> ProjectToSDto => p35 => new ProductCategorySDto()
        {
            Name = p35.Name,
            Description = p35.Description,
            Slug = p35.Slug,
            IsMain = p35.IsMain,
            ParentId = p35.ParentId == null ? default(Guid) : (Guid)p35.ParentId,
            ParentName = p35.Parent != null ? p35.Parent.Name : string.Empty,
            MainImage = p35.Files.FirstOrDefault<ProductCategoryStorageFile>(f => f.IsPrimary) != null ? p35.Files.FirstOrDefault<ProductCategoryStorageFile>(f => f.IsPrimary).FileLocation : (p35.Files.Count > 0 ? p35.Files.FirstOrDefault<ProductCategoryStorageFile>().FileLocation : string.Empty),
            Id = p35.Id,
            CreatedAt = p35.CreatedAt
        };
        
        private static List<ProductCategoryMetaTag> funcMain1(List<MetaTagSDto> p2)
        {
            if (p2 == null)
            {
                return null;
            }
            List<ProductCategoryMetaTag> result = new List<ProductCategoryMetaTag>(p2.Count);
            
            int i = 0;
            int len = p2.Count;
            
            while (i < len)
            {
                MetaTagSDto item = p2[i];
                result.Add(item == null ? null : new ProductCategoryMetaTag()
                {
                    Type = item.Type,
                    Value = item.Value,
                    Id = item.Id,
                    CreatedAt = item.CreatedAt
                });
                i++;
            }
            return result;
            
        }
        
        private static List<ProductCategoryStorageFile> funcMain2(List<StorageFileSDto> p3)
        {
            if (p3 == null)
            {
                return null;
            }
            List<ProductCategoryStorageFile> result = new List<ProductCategoryStorageFile>(p3.Count);
            
            int i = 0;
            int len = p3.Count;
            
            while (i < len)
            {
                StorageFileSDto item = p3[i];
                result.Add(item == null ? null : new ProductCategoryStorageFile()
                {
                    Name = item.Name,
                    FileLocation = item.FileLocation,
                    FileName = item.FileName,
                    IsHeader = item.IsHeader,
                    IsPrimary = item.IsPrimary,
                    FileType = item.FileType,
                    Id = item.Id,
                    CreatedAt = item.CreatedAt
                });
                i++;
            }
            return result;
            
        }
        
        private static ProductCategory funcMain3(Never p6, ProductCategory p7, ProductCategoryLDto p4)
        {
            ProductCategory result = p7 ?? new ProductCategory();
            
            result.Name = p4.ParentName;
            result.Id = p4.ParentId;
            return result;
            
        }
        
        private static List<ProductCategoryMetaTag> funcMain4(List<MetaTagSDto> p8, List<ProductCategoryMetaTag> p9)
        {
            if (p8 == null)
            {
                return null;
            }
            List<ProductCategoryMetaTag> result = new List<ProductCategoryMetaTag>(p8.Count);
            
            int i = 0;
            int len = p8.Count;
            
            while (i < len)
            {
                MetaTagSDto item = p8[i];
                result.Add(item == null ? null : new ProductCategoryMetaTag()
                {
                    Type = item.Type,
                    Value = item.Value,
                    Id = item.Id,
                    CreatedAt = item.CreatedAt
                });
                i++;
            }
            return result;
            
        }
        
        private static List<ProductCategoryStorageFile> funcMain5(List<StorageFileSDto> p10, List<ProductCategoryStorageFile> p11)
        {
            if (p10 == null)
            {
                return null;
            }
            List<ProductCategoryStorageFile> result = new List<ProductCategoryStorageFile>(p10.Count);
            
            int i = 0;
            int len = p10.Count;
            
            while (i < len)
            {
                StorageFileSDto item = p10[i];
                result.Add(item == null ? null : new ProductCategoryStorageFile()
                {
                    Name = item.Name,
                    FileLocation = item.FileLocation,
                    FileName = item.FileName,
                    IsHeader = item.IsHeader,
                    IsPrimary = item.IsPrimary,
                    FileType = item.FileType,
                    Id = item.Id,
                    CreatedAt = item.CreatedAt
                });
                i++;
            }
            return result;
            
        }
        
        private static List<StorageFileSDto> funcMain6(List<ProductCategoryStorageFile> p16)
        {
            if (p16 == null)
            {
                return null;
            }
            List<StorageFileSDto> result = new List<StorageFileSDto>(p16.Count);
            
            int i = 0;
            int len = p16.Count;
            
            while (i < len)
            {
                ProductCategoryStorageFile item = p16[i];
                result.Add(item == null ? null : new StorageFileSDto()
                {
                    Name = item.Name,
                    FileLocation = item.FileLocation,
                    FileName = item.FileName,
                    IsHeader = item.IsHeader,
                    IsPrimary = item.IsPrimary,
                    FileType = item.FileType,
                    Id = item.Id
                });
                i++;
            }
            return result;
            
        }
        
        private static List<MetaTagSDto> funcMain7(List<ProductCategoryMetaTag> p17)
        {
            if (p17 == null)
            {
                return null;
            }
            List<MetaTagSDto> result = new List<MetaTagSDto>(p17.Count);
            
            int i = 0;
            int len = p17.Count;
            
            while (i < len)
            {
                ProductCategoryMetaTag item = p17[i];
                result.Add(item == null ? null : new MetaTagSDto()
                {
                    Type = item.Type,
                    Value = item.Value,
                    Id = item.Id,
                    CreatedAt = item.CreatedAt
                });
                i++;
            }
            return result;
            
        }
        
        private static List<StorageFileSDto> funcMain8(List<ProductCategoryStorageFile> p20, List<StorageFileSDto> p21)
        {
            if (p20 == null)
            {
                return null;
            }
            List<StorageFileSDto> result = new List<StorageFileSDto>(p20.Count);
            
            int i = 0;
            int len = p20.Count;
            
            while (i < len)
            {
                ProductCategoryStorageFile item = p20[i];
                result.Add(item == null ? null : new StorageFileSDto()
                {
                    Name = item.Name,
                    FileLocation = item.FileLocation,
                    FileName = item.FileName,
                    IsHeader = item.IsHeader,
                    IsPrimary = item.IsPrimary,
                    FileType = item.FileType,
                    Id = item.Id
                });
                i++;
            }
            return result;
            
        }
        
        private static List<MetaTagSDto> funcMain9(List<ProductCategoryMetaTag> p22, List<MetaTagSDto> p23)
        {
            if (p22 == null)
            {
                return null;
            }
            List<MetaTagSDto> result = new List<MetaTagSDto>(p22.Count);
            
            int i = 0;
            int len = p22.Count;
            
            while (i < len)
            {
                ProductCategoryMetaTag item = p22[i];
                result.Add(item == null ? null : new MetaTagSDto()
                {
                    Type = item.Type,
                    Value = item.Value,
                    Id = item.Id,
                    CreatedAt = item.CreatedAt
                });
                i++;
            }
            return result;
            
        }
        
        private static ProductCategory funcMain10(Never p30, ProductCategory p31, ProductCategorySDto p28)
        {
            ProductCategory result = p31 ?? new ProductCategory();
            
            result.Name = p28.ParentName;
            result.Id = p28.ParentId;
            return result;
            
        }
        
        private static bool funcMain11(ProductCategoryStorageFile f)
        {
            return f.IsPrimary;
        }
        
        private static bool funcMain12(ProductCategoryStorageFile f)
        {
            return f.IsPrimary;
        }
    }
}