﻿namespace Netina.Repository.Handlers.SubProducts;

public class CreateSubProductCommandHandler(IRepositoryWrapper repositoryWrapper,
    ICurrentUserService currentUserService) : IRequestHandler<CreateSubProductCommand,Guid>
{
    public async Task<Guid> Handle(CreateSubProductCommand request, CancellationToken cancellationToken)
    {

        if (currentUserService.UserId == null)
            throw new BaseApiException(ApiResultStatusCode.UnAuthorized, "User id is wrong");

        if (!Guid.TryParse(currentUserService.UserId, out Guid userId))
            throw new BaseApiException(ApiResultStatusCode.UnAuthorized, "User id is wrong");

        var parent = await repositoryWrapper.SetRepository<Product>().TableNoTracking
            .FirstOrDefaultAsync(p => p.Id == request.ParentId, cancellationToken);
        if (parent == null)
            throw new BaseApiException(ApiResultStatusCode.NotFound, "Parent product not found");

        var subProduct = SubProduct.Create(
            request.ParentId,
            request.Diversity,
            request.DiversityValue,
            request.DiversityDescription,
            $"{parent.PersianName} - {request.DiversityDescription}",
            parent.EnglishName,
            parent.Summery,
            parent.Slug,
            parent.ExpertCheck,
            parent.Tags,
            parent.Warranty,
            parent.BeDisplayed,
            request.Cost,
            request.PackingCost,
            request.HasExpressDelivery,
            request.Stock,
            request.MaxOrderCount,
            parent.BrandId,
            parent.CategoryId,
            userId);

        foreach (var file in request.Files)
            subProduct.AddFile(file.Name, file.FileLocation, file.FileName, file.IsHeader, file.IsPrimary, file.FileType);

        repositoryWrapper.SetRepository<SubProduct>().Add(subProduct);
        await repositoryWrapper.SaveChangesAsync(cancellationToken);
        return subProduct.Id;
    }
}